// -*- C++ -*-
//
// emdump.cc
// 
// Author:  Ken Sakashita
// Created: Wed Aug 21 15:32:01 JST 2002
//
// $Id: emdump.cc,v 1.1 2002/08/30 02:12:14 kensh Exp $
//
// Revision History
//
// $Log: emdump.cc,v $
// Revision 1.1  2002/08/30 02:12:14  kensh
//
//
// New release. event dump tool.
//
//

#include <iostream>
#include <list>
#include <signal.h>
#include <string>
#include <stdlib.h>
#include "MidasManager/MidasBank.h"
#include "MidasManager/MidasFileIO.h"

// globals
MidasBank mbank;

//////// user code is defined in the user_ana.cc
extern void  user_init();
extern void  user_begin_of_run( int );
extern int   user_process_event( EVENT_HEADER* ,void* , MidasBank& );
extern void  user_end_of_run( int );
extern void  user_term();

////////
// usage
void
usage( int argc, char* argv[] )
{
  std::cout << "usage : " << argv[0] << " [options] filename"
	    << std::endl
	    << "  options: " << std::endl
	    << "     -i <event id>       ...Event ID you want to dump (default = 0)"
	    << std::endl
	    << "     -dump               ...Dump all words in each bank"
	    << std::endl
	    << "     -n <#event>         ...# of events you want to dump"
	    << std::endl
	    << "     -skip <skip #event> ...# of skip events "
	    << std::endl;

}

typedef struct
{
  int   Eventid;
  bool  Dumpevent;
  int   Nevent;
  int   Skipevent;

} DUMP_OPTIONS;
DUMP_OPTIONS options;

////////
// main
int
main( int argc, char* argv[] )
{
  // initialize options
  options.Eventid   = 0;
  options.Dumpevent = false;
  options.Nevent    = -1;
  options.Skipevent = 0;
  // input file
  std::string ifname;

  // options
  if( argc < 2 ){
	usage( argc,argv );
	return( 1 );
  }
  for( int i = 1; i < argc; i++){
    if( argv[i][0] == '-' ){
      if( strcmp( argv[i], "-i") == 0 ){
	options.Eventid = strtol( argv[++i],NULL,0 );
      }
      else if( strcmp( argv[i], "-dump" ) == 0){
	options.Dumpevent = true;
      }
      else if( strcmp( argv[i], "-n" ) == 0){
	options.Nevent = strtol( argv[++i],NULL,0 );
      }
      else if( strcmp( argv[i], "-skip" ) == 0){
	options.Skipevent = strtol( argv[++i],NULL,0 );
      }
      else{
	std::cout << " unknown option : " << argv[i] << std::endl;
	usage( argc,argv );
	return( 1 );
      }
    }
    else{
      // input file
      ifname = std::string( argv[i] );
    }
  }

  // call user init routine
  user_init();

  // open data file
  MidasFileIO mfile( ifname, O_RDONLY|O_LARGEFILE );

  if ( mfile.good() ){
    mfile.bank( mbank );
    std::cout << "Opening input file : " << ifname << std::endl;
    
    // event bank
    EVENT_HEADER*  pheader = mbank.eventHeader();
    void*          pevent  = mbank.eventBuffer();
    
    // Begin_of_run record
    int  run_number = -1;
    if ( mfile.readBOR() ){
      run_number = pheader->serial_number;
      std::cout << "Run = " << run_number << std::endl;
      
      // call user begin_of_run routine
      user_begin_of_run( run_number );
    }
    
    // Process Event
    mfile.skipEvent( 0,options.Skipevent );
    for( int i = 0; i < options.Nevent || options.Nevent < 0; i++){
      int  id = mfile.readEvent(0);
      if ( (id & 0x8000) == 0x8000 ){
	// Special event (BOR,EOR,MES)
	if ( id == EVENTID_BOR ){
	  // call user begin_of_run routine
	  user_begin_of_run( run_number );
	}
	else if ( pheader->event_id == EVENTID_EOR ){
	  // call user end_of_run routine
	  user_end_of_run( run_number );
	}
	
      }
      else if ( id > 0 ){
	if ( user_process_event( pheader, pevent, mbank ) < 0 )
	  break;
      }
      else{
	// end of file
	break;
      }
    }
    
  }
  else{
    std::cout << "No such file : " << ifname << std::endl;
  }
  
  // call user term routine
  user_term();

  return( 0 );
}

