// -*- C++ -*-
//
// drawdet.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Sep 17 13:36:24 JST 2002
//
// $Id: drawdet.cc,v 1.2 2002/10/07 13:50:51 yamaga Exp $
//
// Revision History
//
// $Log: drawdet.cc,v $
// Revision 1.2  2002/10/07 13:50:51  yamaga
// Adc and scale-factor corrected.
//
// Revision 1.1  2002/09/27 12:37:02  yamaga
// new package
//
//
//

#include <iostream>
#include <iomanip>
#include <string>
#include <math.h>
#include <time.h>
#include "E391Detector/E391Detector.h"
#include "HigzManager/HigzManager.h"

extern E391Detector&  e391det;
extern int     WkType; 
extern double  RotAngle;
extern double  AdcCutL;
extern double  AdcCutH;
extern std::string   Psfile;


#define  NWPAWC  1000000
int  pawc_[NWPAWC];

HigzManager*  hg;

//
typedef struct{
  Hep3Vector  pos;
  Hep3Vector  p[4];
} DETSHAPE;

DETSHAPE  csidet[576];
DETSHAPE  sanddet[24];
DETSHAPE  cc03det[6];
DETSHAPE  chgvdet[36];
DETSHAPE  c6det[2];
DETSHAPE  cosmdet[6];

static double  ScreenSize = 130;

// inline
inline double
logScaleFactor( double val, double order )
{
  double  sc = 1 + log(val) / order;
  if ( sc < 0 )
    sc = 0;
  return( sc );
}


// 
void
initDisplay()
{
  HigzManager::init( NWPAWC );
  hg = new HigzManager( 1,WkType );
  hg->SetWindow( -ScreenSize,ScreenSize, -ScreenSize,ScreenSize );
  hg->DoubleBuffer();

  // CsI
  int  ncsi[] = {  7,13,15,19,21,21,23,25,25,25,27,27,27,27,
                   27,27,27,25,25,25,23,21,21,19,15,13,7     };
  int  ct = 0;
  for( int iy=0;iy<27;iy++ ){
    for( int ix=0;ix<ncsi[iy];ix++ ){
      double  x = ix - 13 + (27-ncsi[iy])/2;
      double  y = iy - 13;
      double  z = 0;
      if ( -2<=x && x <=2 && -2<=y && y<=2 ){
        // KTeV CsI
        int  *kx,*ky;
        int  nk = 0;
        if ( y == -2 && x == -2 ){
          static int  kxcsi[] = { -3,-2,-1, 0, 1, 2, 3 }; 
          static int  kycsi[] = { -3,-3,-3,-3,-3,-3,-3 };
          kx = kxcsi;
          ky = kycsi;
          nk = 7;
        }
        else if ( y == -1 && x == -2 ){
          int  kxcsi[] = { -3,-3, 3, 3 }; 
          int  kycsi[] = { -2,-1,-2,-1 };
          kx = kxcsi;
          ky = kycsi;
          nk = 4;
        }
        else if ( y == 0 && x == -2 ){
          int  kxcsi[] = { -3, 3 }; 
          int  kycsi[] = {  0, 0 };
          kx = kxcsi;
          ky = kycsi;
          nk = 2;
        }
        else if ( y == 1 && x == -2 ){
          int  kxcsi[] = { -3,-3, 3, 3 }; 
          int  kycsi[] = {  1, 2, 1, 2 };
          kx = kxcsi;
          ky = kycsi;
          nk = 4;
        }
        else if ( y == 2 && x == -2 ){
          static int  kxcsi[] = { -3,-2,-1, 0, 1, 2, 3 }; 
          static int  kycsi[] = {  3, 3, 3, 3, 3, 3, 3 };
          kx = kxcsi;
          ky = kycsi;
          nk = 7;
        }
        for( int ki=0;ki<nk;ki++ ){
          double  x = kx[ki];
          double  y = ky[ki];
          double  z = 0;
          double  dxy = 5;
          csidet[ct].pos = Hep3Vector( x*dxy, y*dxy, z );
          Hep3Vector  v1( dxy/2, dxy/2, 0 );
          Hep3Vector  v2( dxy/2,-dxy/2, 0 );
          csidet[ct].p[0] = csidet[ct].pos - v1;
          csidet[ct].p[1] = csidet[ct].pos - v2;
          csidet[ct].p[2] = csidet[ct].pos + v1;
          csidet[ct].p[3] = csidet[ct].pos + v2;
          ct++;
        }
      }
      else{
        double  dxy = 7;
        csidet[ct].pos = Hep3Vector( x*dxy, y*dxy, z );
        Hep3Vector  v1( dxy/2, dxy/2, 0 );
        Hep3Vector  v2( dxy/2,-dxy/2, 0 );
        csidet[ct].p[0] = csidet[ct].pos - v1;
        csidet[ct].p[1] = csidet[ct].pos - v2;
        csidet[ct].p[2] = csidet[ct].pos + v1;
        csidet[ct].p[3] = csidet[ct].pos + v2;
        csidet[ct].pos = Hep3Vector( x*dxy, y*dxy, z );
        ct++;
      }
    }
  }

  // Sandwich
  //int  sandnum[] = {  0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10,11,
  //                    12,13,14,15,16,17,18,19,20,21,22,23, };
  int  sandnum[] = {  0, 8,16,20,12, 4,   5,13,21,17, 9, 1,
                      3,11,19,23,15, 7,   6,14,22,18,10, 2, };
  int  sandpos[24];
  for( int i=0;i<24;i++ ){
    for( int j=0;j<24;j++ ){
      if ( sandnum[j] == i )
        sandpos[i] = j;
    }
  }
  double  sand_x[]    = { 13,12,11 };
  double  sand_y[]    = {  4, 7, 8 };
  double  sand_dx[] = { 0.6, 0.4, 0.8 };
  double  sand_dy[] = { 1.7, 0.7, 1.0 };
  double  sand_dxy = 7;
  ct = 0;
  for( int i=0;i<3;i++ ){
    // ++
    int  n = sandpos[ct++];
    int  n0 = n;
    sanddet[n].p[0] = Hep3Vector( (sand_x[i]-.5)*sand_dxy,
                                  (sand_y[i]-.5)*sand_dxy,
                                  0 );
    sanddet[n].p[1] = Hep3Vector( (sand_x[i]-.5+sand_dx[i])*sand_dxy,
                                  (sand_y[i]-.5           )*sand_dxy,
                                  0 );
    sanddet[n].p[2] = Hep3Vector( (sand_x[i]-.5           )*sand_dxy,
                                  (sand_y[i]-.5+sand_dy[i])*sand_dxy,
                                  0 );
    sanddet[n].p[3] = sanddet[n].p[2];
    sanddet[n].pos = (sanddet[n].p[0]+sanddet[n].p[1]+sanddet[n].p[2])/3;
    // -+
    n = sandpos[ct++];
    sanddet[n].p[0]= Hep3Vector(-sanddet[n0].p[0].x(), sanddet[n0].p[0].y(),0);
    sanddet[n].p[1]= Hep3Vector(-sanddet[n0].p[1].x(), sanddet[n0].p[1].y(),0);
    sanddet[n].p[2]= Hep3Vector(-sanddet[n0].p[2].x(), sanddet[n0].p[2].y(),0);
    sanddet[n].p[3]= Hep3Vector(-sanddet[n0].p[3].x(), sanddet[n0].p[3].y(),0);
    sanddet[n].pos = (sanddet[n].p[0]+sanddet[n].p[1]+sanddet[n].p[2])/3;
    // +-
    n = sandpos[ct++];
    sanddet[n].p[0]= Hep3Vector( sanddet[n0].p[0].x(),-sanddet[n0].p[0].y(),0);
    sanddet[n].p[1]= Hep3Vector( sanddet[n0].p[1].x(),-sanddet[n0].p[1].y(),0);
    sanddet[n].p[2]= Hep3Vector( sanddet[n0].p[2].x(),-sanddet[n0].p[2].y(),0);
    sanddet[n].p[3]= Hep3Vector( sanddet[n0].p[3].x(),-sanddet[n0].p[3].y(),0);
    sanddet[n].pos = (sanddet[n].p[0]+sanddet[n].p[1]+sanddet[n].p[2])/3;
    // --
    n = sandpos[ct++];
    sanddet[n].p[0]= Hep3Vector(-sanddet[n0].p[0].x(),-sanddet[n0].p[0].y(),0);
    sanddet[n].p[1]= Hep3Vector(-sanddet[n0].p[1].x(),-sanddet[n0].p[1].y(),0);
    sanddet[n].p[2]= Hep3Vector(-sanddet[n0].p[2].x(),-sanddet[n0].p[2].y(),0);
    sanddet[n].p[3]= Hep3Vector(-sanddet[n0].p[3].x(),-sanddet[n0].p[3].y(),0);
    sanddet[n].pos = (sanddet[n].p[0]+sanddet[n].p[1]+sanddet[n].p[2])/3;
    // x<->y, ++
    n = sandpos[ct++];
    sanddet[n].p[0]= Hep3Vector( sanddet[n0].p[0].y(), sanddet[n0].p[0].x(),0);
    sanddet[n].p[1]= Hep3Vector( sanddet[n0].p[1].y(), sanddet[n0].p[1].x(),0);
    sanddet[n].p[2]= Hep3Vector( sanddet[n0].p[2].y(), sanddet[n0].p[2].x(),0);
    sanddet[n].p[3]= Hep3Vector( sanddet[n0].p[3].y(), sanddet[n0].p[3].x(),0);
    sanddet[n].pos = (sanddet[n].p[0]+sanddet[n].p[1]+sanddet[n].p[2])/3;
    // x<->y, -+
    n = sandpos[ct++];
    sanddet[n].p[0]= Hep3Vector(-sanddet[n0].p[0].y(), sanddet[n0].p[0].x(),0);
    sanddet[n].p[1]= Hep3Vector(-sanddet[n0].p[1].y(), sanddet[n0].p[1].x(),0);
    sanddet[n].p[2]= Hep3Vector(-sanddet[n0].p[2].y(), sanddet[n0].p[2].x(),0);
    sanddet[n].p[3]= Hep3Vector(-sanddet[n0].p[3].y(), sanddet[n0].p[3].x(),0);
    sanddet[n].pos = (sanddet[n].p[0]+sanddet[n].p[1]+sanddet[n].p[2])/3;
    // x<->y, +-
    n = sandpos[ct++];
    sanddet[n].p[0]= Hep3Vector( sanddet[n0].p[0].y(),-sanddet[n0].p[0].x(),0);
    sanddet[n].p[1]= Hep3Vector( sanddet[n0].p[1].y(),-sanddet[n0].p[1].x(),0);
    sanddet[n].p[2]= Hep3Vector( sanddet[n0].p[2].y(),-sanddet[n0].p[2].x(),0);
    sanddet[n].p[3]= Hep3Vector( sanddet[n0].p[3].y(),-sanddet[n0].p[3].x(),0);
    sanddet[n].pos = (sanddet[n].p[0]+sanddet[n].p[1]+sanddet[n].p[2])/3;
    // x<->y, --
    n = sandpos[ct++];
    sanddet[n].p[0]= Hep3Vector(-sanddet[n0].p[0].y(),-sanddet[n0].p[0].x(),0);
    sanddet[n].p[1]= Hep3Vector(-sanddet[n0].p[1].y(),-sanddet[n0].p[1].x(),0);
    sanddet[n].p[2]= Hep3Vector(-sanddet[n0].p[2].y(),-sanddet[n0].p[2].x(),0);
    sanddet[n].p[3]= Hep3Vector(-sanddet[n0].p[3].y(),-sanddet[n0].p[3].x(),0);
    sanddet[n].pos = (sanddet[n].p[0]+sanddet[n].p[1]+sanddet[n].p[2])/3;
  }

  // CC03
  double  cc03_x[] = {  1.5,   0, -1.5, -1.5,  0,    1.5 };
  double  cc03_y[] = {  1,   1.5,  1,   -1,   -1.5, -1   };
  double  cc03_dxy = 25./4.;
  for( int i=0;i<6;i++ ){
    cc03det[i].pos = Hep3Vector( cc03_x[i]*cc03_dxy, cc03_y[i]*cc03_dxy, 0 );
    if ( i==1 || i == 4 ){
      Hep3Vector  vx( cc03_dxy, 0, 0 );
      Hep3Vector  vy( 0, cc03_dxy/2, 0 );
      cc03det[i].p[0] = cc03det[i].pos - vx + vy;
      cc03det[i].p[1] = cc03det[i].pos - vx - vy;
      cc03det[i].p[2] = cc03det[i].pos + vx - vy;
      cc03det[i].p[3] = cc03det[i].pos + vx + vy;
    }
    else{
      Hep3Vector  vx( cc03_dxy/2, 0, 0 );
      Hep3Vector  vy( 0, -cc03_dxy, 0 );
      cc03det[i].p[0] = cc03det[i].pos - vx + vy;
      cc03det[i].p[1] = cc03det[i].pos - vx - vy;
      cc03det[i].p[2] = cc03det[i].pos + vx - vy;
      cc03det[i].p[3] = cc03det[i].pos + vx + vy;
    }
  }
  
  // CHGV
  for( int i=0;i<32;i++ ){
    double  r  = 102;
    double  ri = r*.98;
    double  ro = r*1.03;
    double  ang = 2*M_PI*i/32.;
    double  al  = 2*M_PI*(i-.5)/32.;
    double  ah  = 2*M_PI*(i+.5)/32.;
    chgvdet[i].pos  = Hep3Vector( r*cos(ang),r*sin(ang), 0 );
    chgvdet[i].p[0] = Hep3Vector( ri*cos(al),ri*sin(al), 0 );
    chgvdet[i].p[1] = Hep3Vector( ri*cos(ah),ri*sin(ah), 0 );
    chgvdet[i].p[2] = Hep3Vector( ro*cos(ah),ro*sin(ah), 0 );
    chgvdet[i].p[3] = Hep3Vector( ro*cos(al),ro*sin(al), 0 );
  }
  for( int i=0;i<4;i++ ){
    double  r  = 25./4. - 1;
    double  ro = 25./4.*sqrt(2.);
    double  ri = ro - 2*sqrt(2.);
    double  ang = 2*M_PI*i/4.;
    double  al  = 2*M_PI*(i-.5)/4.;
    double  ah  = 2*M_PI*(i+.5)/4.;
    chgvdet[i+32].pos  = Hep3Vector( r*cos(ang),r*sin(ang), 0 );
    chgvdet[i+32].p[0] = Hep3Vector( ri*cos(al),ri*sin(al), 0 );
    chgvdet[i+32].p[1] = Hep3Vector( ri*cos(ah),ri*sin(ah), 0 );
    chgvdet[i+32].p[2] = Hep3Vector( ro*cos(ah),ro*sin(ah), 0 );
    chgvdet[i+32].p[3] = Hep3Vector( ro*cos(al),ro*sin(al), 0 );
  }

  // C6
  double  c6r  = 5;
  double  c6ri = 3;
  double  c6ro = 8;
  Hep3Vector  c6center( -90,-90,0 );
  //
  c6det[0].pos  = Hep3Vector( c6r,0,0 ) + c6center;
  c6det[0].p[0] = c6center; // org
  c6det[0].p[1] = Hep3Vector( 0,0,1 ); // norm
  c6det[0].p[2] = Hep3Vector( 0,-1,0 ); // baseline
  c6det[0].p[3] = Hep3Vector( c6ri,c6ro,0 ); // rmin,rmax
  //
  c6det[1].pos  = Hep3Vector( -c6r,0,0 ) + c6center;
  c6det[1].p[0] = c6center; // org
  c6det[1].p[1] = Hep3Vector( 0,0,1 ); // norm
  c6det[1].p[2] = Hep3Vector( 0,1,0 ); // baseline
  c6det[1].p[3] = Hep3Vector( c6ri,c6ro,0 ); // rmin,rmax

  // Cosmic
  double  cosm_x[] = {  1,   -1,     1,   -1,  1,-1 };
  double  cosm_y[] = {  1,    1,  1.03, 1.03, -1,-1 };
  double  cosm_cx = 50;
  double  cosm_cy = 110;
  double  cosm_dx = 50;
  double  cosm_dy = 1;
  for( int i=0;i<6;i++ ){
    Hep3Vector  vx( cosm_dx,0,0 );
    Hep3Vector  vy( 0,cosm_dy,0 );
    cosmdet[i].pos  = Hep3Vector( cosm_x[i]*cosm_cx,  cosm_y[i]*cosm_cy, 0 );
    cosmdet[i].p[0] = cosmdet[i].pos + vx + vy;
    cosmdet[i].p[1] = cosmdet[i].pos + vx - vy;
    cosmdet[i].p[2] = cosmdet[i].pos - vx - vy;
    cosmdet[i].p[3] = cosmdet[i].pos - vx + vy;
  }

  if ( !Psfile.empty() ){
    hg->OpenPsFile( Psfile );
  }
}


// 
void
drawDetector( int runnum, int spillnum, int eventnum, int timestamp )
{
  hg->ResizeWindow();
  hg->ClearScreen();
  hg->Igset( "FAIS",FAIS_HOLLOW );
  hg->Igset( "FACI",BLACK );

  if ( RotAngle != 0 ){
    hg->Zoom( Hep3Vector( 0,0,0 ),400 );
    static double  rot = 0;
    hg->Rotation( "y",rot,0 );
    rot = fmod( rot + RotAngle, 360 );
  }

  // title
  char  s[128];
  sprintf( s,"Run %d  Spill %d  Event %d",
           runnum,spillnum,eventnum );
  hg->Igset( "TXFP",-50 );
  hg->Igset( "CHHE",6 );
  hg->Igset( "TXAL",0 );
  hg->Puts( -ScreenSize*.90,ScreenSize*.93,s );

  // time
  hg->Puts( ScreenSize*.2,ScreenSize*.93,
            asctime(localtime((time_t*)&timestamp)) );

  // label
  hg->Igset( "TXFP",-130  );
  hg->Igset( "CHHE",2 );
  hg->Igset( "TXAL",23 );

  // CsI
  for( int i=0;i<576;i++ ){
    hg->Polygon( 4,csidet[i].p );
    char  s[8];
    sprintf( s,"%d",i );
    hg->Puts( csidet[i].pos,s );
  }

  // Sand
  for( int i=0;i<24;i++ ){
    hg->Polygon( 4,sanddet[i].p );
    char  s[8];
    sprintf( s,"%d",i );
    hg->Puts( sanddet[i].pos*1.03,s );
  }

  // CC03
  for( int i=0;i<6;i++ ){
    hg->Polygon( 4,cc03det[i].p );
    char  s[8];
    sprintf( s,"%d",i );
    hg->Puts( cc03det[i].pos,s );
  }

  // CHGV
  for( int i=0;i<36;i++ ){
    hg->Polygon( 4,chgvdet[i].p );
    char  s[8];
    sprintf( s,"%d",i );
    hg->Puts( chgvdet[i].pos,s );
  }

  // C6
  for( int i=0;i<2;i++ ){
    hg->Arc( c6det[i].p[0],c6det[i].p[1],c6det[i].p[2],
             c6det[i].p[3].x(),c6det[i].p[3].y(),
             0,180 );
    char  s[8];
    sprintf( s,"%d",i );
    hg->Puts( c6det[i].pos,s );
  }

  // Cosmic
  for( int i=0;i<6;i++ ){
    hg->Polygon( 4,cosmdet[i].p );
    char  s[8];
    sprintf( s,"%d",i );
    hg->Puts( cosmdet[i].pos,s );
  }

  //
  //hg->Update();
}


// 
void
drawHit( int runnum, int spillnum, int eventnum, int trigbit )
{
  // Trigger
  static  char*  trgstr[]={
    "OnSpill","Clock","Cosmic","Xe","Ncluster","NXcluster","",""
  };
  static  int  trgcolor[]={
    BLACK,GREEN,RED,GREEN,RED,RED,RED,RED
  };
  
  int  hit_color = BLACK;
  std::string  s_trig;
  char ss[64];
  sprintf( ss,"Trig = 0x%04x : ",trigbit & 0xffff );
  s_trig += ss;
  for( int i=0;i<8;i++ ){
    if ( trigbit & (1<<i) ){
      s_trig += trgstr[i] + std::string(", ");
      hit_color = trgcolor[i];
    }
  }
  hg->Igset( "TXFP",-50 );
  hg->Igset( "CHHE",4.8);
  hg->Igset( "TXAL",0 );
  hg->Puts( -ScreenSize*.90,-ScreenSize*.93,s_trig );

  //
  // CsI
  for( int i=0;i<576;i++ ){
    int  dt = e391det.CsI(i).adc();
    if ( dt > 0 ){
      int  range = (dt >> 16) & 0x1;
      int  adc = dt & 0x0fff;
      if ( ( range == 0 && adc > AdcCutL ) ||
           ( range >  0 && adc > AdcCutH )  ){
        //std::cout << "CsI" << i << "=" 
        //          << e391det.CsI(i).adc()
        //          << std::endl;
        if ( range )
          adc <<= 3; 
        hg->Igset( "FAIS",FAIS_SOLID );
        hg->Igset( "FACI",hit_color );
        double  sf = logScaleFactor( (double)adc/0x7fff,5 );
        Hep3Vector pp[4];
        for( int c=0;c<4;c++ ){
          pp[c] = csidet[i].pos + (csidet[i].p[c] - csidet[i].pos)*sf;
          //std::cout << csidet[i].p[c] << pp[c] << std::endl;
        }
        hg->Polygon( 4,pp );
      }
    }
  }

  // Sand
  for( int i=0;i<24;i++ ){
    int  dt = e391det.Sandwich(i).adc();
    if ( dt > 0 ){
      int  range = (dt >> 16) & 0x1;
      int  adc = dt & 0x0fff;
      if ( ( range == 0 && adc > AdcCutL ) ||
           ( range >  0 && adc > AdcCutH )  ){
        if ( range )
          adc <<= 3; 
        hg->Igset( "FAIS",FAIS_SOLID );
        hg->Igset( "FACI",hit_color );
        double  sf = logScaleFactor( (double)adc/0x7fff,5 );
        Hep3Vector pp[4];
        for( int c=0;c<4;c++ ){
          pp[c] = sanddet[i].pos + (sanddet[i].p[c] - sanddet[i].pos)*sf;
        }
        hg->Polygon( 4,pp );
      }
    }
  }

  // CC03
  for( int i=0;i<6;i++ ){
    int  dt = e391det.CC03(i).adc();
    if ( dt > 0 ){
      int  range = (dt >> 16) & 0x1;
      int  adc = dt & 0x0fff;
      if ( (range == 0 && adc > AdcCutL ) ||
           (range >  0 && adc > AdcCutH )  ){
        if ( range )
          adc <<= 3; 
        hg->Igset( "FAIS",FAIS_SOLID );
        hg->Igset( "FACI",hit_color );
        double  sf = logScaleFactor( (double)adc/0x7fff,5 );
        Hep3Vector pp[4];
        for( int c=0;c<4;c++ ){
          pp[c] = cc03det[i].pos + (cc03det[i].p[c] - cc03det[i].pos)*sf;
        }
        hg->Polygon( 4,pp );
      }
    }
  }

  // CHGV
  for( int i=0;i<36;i++ ){
    int  dt = e391det.ChargeVeto(i).adc();
    if ( dt > 0 ){
      int  range = (dt >> 16) & 0x1;
      int  adc = dt & 0x0fff;
      if ( (range == 0 && adc > AdcCutL ) ||
           (range >  0 && adc > AdcCutH )  ){
        if ( range )
          adc <<= 3; 
        hg->Igset( "FAIS",FAIS_SOLID );
        hg->Igset( "FACI",hit_color );
        double  sf = logScaleFactor( (double)adc/0x7fff,5 );
        Hep3Vector pp[4];
        for( int c=0;c<4;c++ ){
          pp[c] = chgvdet[i].pos + (chgvdet[i].p[c] - chgvdet[i].pos)*sf;
        }
        hg->Polygon( 4,pp );
      }
    }
  }
  
  // C6
  for( int i=0;i<2;i++ ){
    int  dt = e391det.C6(i).adc();
    if ( dt > 0 ){
      int  range = (dt >> 16) & 0x1;
      int  adc = dt & 0x0fff;
      if ( (range == 0 && adc > AdcCutL ) ||
           (range >  0 && adc > AdcCutH )  ){
        if ( range )
          adc <<= 3; 
        hg->Igset( "FAIS",FAIS_SOLID );
        hg->Igset( "FACI",hit_color );
        double  sf = logScaleFactor( (double)adc/0x7fff,5 );
        hg->Arc( c6det[i].p[0],c6det[i].p[1],c6det[i].p[2],
                 c6det[i].p[3].x(),c6det[i].p[3].y()*sf,
                 90*(1-sf),90*(1+sf) );
      }
    }
  }
  
  // Cosmic
  for( int i=0;i<6;i++ ){
    int  dt = e391det.CosmicCounter(i).adc();
    if ( dt > 0 ){
      int  range = (dt >> 16) & 0x1;
      int  adc = dt & 0x0fff;
      //if ( (range == 0 && adc > AdcCutL ) ||
      //     (range >  0 && adc > AdcCutH )  ){
      if ( ( range == 0 && adc > 800 ) ||
           ( range >  0 && adc > 800 )  ){
        if ( range )
          adc <<= 3; 
        hg->Igset( "FAIS",FAIS_SOLID );
        hg->Igset( "FACI",hit_color );
        double  sf = logScaleFactor( (double)adc/0x7fff,5 );
        Hep3Vector pp[4];
        for( int c=0;c<4;c++ ){
          pp[c] = cosmdet[i].pos + (cosmdet[i].p[c] - cosmdet[i].pos)*sf;
        }
        hg->Polygon( 4,pp );
      }
    }
  }
  
  //
  hg->Update();

}


//
void
termDisplay()
{
  if ( !Psfile.empty() ){
    hg->ClosePsFile();
  }
  delete hg;
  HigzManager::term();
}

