// -*- C++ -*-
//
// user_ana.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Sep 17 13:36:24 JST 2002
//
// $Id: user_ana.cc,v 1.3 2002/10/14 06:20:31 yamaga Exp $
//
// Revision History
//
// $Log: user_ana.cc,v $
// Revision 1.3  2002/10/14 06:20:31  yamaga
// Added -pause option for offline mode.
//
// Revision 1.2  2002/10/07 13:53:39  yamaga
// -usleep option applicable even for online mode.
//
// Revision 1.1  2002/09/27 12:37:02  yamaga
// new package
//
//
//

#include <iostream>
#include <iomanip>
#include <unistd.h>
#include "E391Detector/E391Detector.h"

#define  TRIG_SPILLON  0x01
#define  TRIG_CLOCK    0x02
#define  TRIG_COSMIC   0x04
#define  TRIG_XE       0x08
#define  TRIG_NCLUS    0x10

//
extern E391Detector&  e391det;
extern bool    Pause;
extern double  Freq;
extern int     Interval;
extern int     Usleep;
extern unsigned int  TriggerBit;

// drawdet.cc
void  initDisplay();
void  drawDetector( int,int,int,int );
void  drawHit( int,int,int,int );
void  termDisplay();


// init routine
void
user_init()
{
  // Histogram definition
  initDisplay();
}


// begin_of_run routine
void
user_begin_of_run( int run_number )
{
  // 
}


// process_event routine
void
user_process_event( int run_number, int spill_number,
                    int event_number, int trigger_bit )
{
  static int  ct = 0;
  if ( ++ct % 100 == 0 ){
    std::cout << "count=" << ct << " Run" << run_number 
              << " spill=" << spill_number << " Event " << event_number
              << std::endl;
  }
  
  if ( Interval <= 0 || ct % Interval == 1 ){
    int  tm = e391det.eventTime();
    drawDetector( run_number, spill_number, event_number, tm );
    drawHit( run_number, spill_number, event_number,trigger_bit );
    if ( Pause ){
      std::cout << "(hit 'enter' key to proceed next event)";
      std::cout.flush();
      getchar();
    }
    else
      usleep( Usleep );
  }
}


// process_event routine
void
user_process_event_exp( HNDLE hbuf,HNDLE request_id,
                        EVENT_HEADER* pheader, void* pevent )
{
  /* ODB structures */
  extern RUNINFO  runinfo;
  
  int  spillnum = pheader->serial_number;

  //
  static int  RunNumber = 0;
  if ( RunNumber != runinfo.run_number ){
    RunNumber = runinfo.run_number;
    e391det.setRunNumber( RunNumber );
    // get DB for this run
    e391det.getDB();
    //e391det.getDB( 1 );
  }

  //
  e391det.setEvent( pheader, pevent );
  DWORD*  pdata;
  int  nb = e391det.bank( "NGF0",&pdata );

  if ( nb > 0 ){
    std::cout << "Run" << runinfo.run_number
              << " spill=" << spillnum
              << " nevent=" << pdata[0]
              << std::endl;

    int  nevent = pdata[0];
    int  nskip  = (int)(nevent/Freq) / 4 + 1;
    int  nev = 1;
    int  ct = 0;
    while( 1 ){
      if ( e391det.unpack( nev ) == 0 ){
        //
        int  trig = e391det.triggerBit(nev);
        int  tm   = e391det.eventTime();
        if ( TriggerBit ){
          if ( trig & TriggerBit ){
            if ( Interval <=0 || ct++ % Interval == 0 ){
              drawDetector( RunNumber, spillnum, nev, tm );
              drawHit( RunNumber, spillnum, nev, trig );
              if ( nev + 1 <= nevent )
                usleep( Usleep );
            }
          }
          nev++;
        }
        else{
          drawDetector( RunNumber, spillnum, nev, tm );
          drawHit( RunNumber, spillnum, nev, trig );
          if ( Interval <= 0 ){
            if ( (nev += nskip) <= nevent ){
              if ( Usleep <= 0 )
                usleep( (int)((800000)/Freq) );
              else
                usleep( Usleep );
            }
          }
          else{
            if ( (nev += Interval) <= nevent )
              usleep( Usleep );
          }
        }
      }
      else{
        break;
      }
    }
  }
}


// end_of_run routine
void
user_end_of_run( int run_number )
{
  //
}


// term routine
void
user_term()
{
  termDisplay();
  //
}


