// -*- C++ -*-
//
// onlineMonirot.cc
//
// Author:  Ken Sakashita
// Created: Mon Oct  7 22:51:58 JST 2002
//
// $Id: onlineMonitor.cc,v 1.1 2002/10/07 14:04:50 kensh Exp $
//
// $Log: onlineMonitor.cc,v $
// Revision 1.1  2002/10/07 14:04:50  kensh
//
//
// Add new package "onlineMonitor"
//
//
//

#include <iostream>
#include <list>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include "midas.h"
#include "E391Detector/E391Detector.h"
#include "MidasManager/MidasManager.h"


// globals
int Debug = 0;
unsigned int  TriggerBit = 0;
std::string  Psfile;
E391Detector&  e391det = E391Detector::instance();

/* ODB structures */
RUNINFO  runinfo;


//////// user code is defined in the user_ana.cc
extern void  user_init();
extern void  user_begin_of_run( int runnum );
extern void  user_process_event( int runnum,int spillnum,
                                 int eventnum,int trigbit );
extern void  user_process_event_exp( HNDLE hbuf,HNDLE request_id,
                                     EVENT_HEADER* pheader, void* pevent );
extern void  user_end_of_run( int runnum );
extern void  user_term();


////////
// usage
static void
usage( int argc, char* argv[] )
{
  std::cout << "usage : " << argv[0] << " [options]"
            << std::endl 
            << "  options:" << std::endl
            << "    -h <hostname> (connect to the running experiment.)"
            << std::endl;
}


////////
// main
int
main( int argc, char* argv[] )
{
  if ( argc < 2 ){
    usage( argc,argv );
    return( 1 );
  }

  // command parser
  int  Skipevent = 0;
  int  Nevent  = -1;
  int  runFrom = 0;
  int  runTo   = 0;
  std::list<std::string>  iflist;
  std::list<std::string>  iftmp;
  char  exphost[64];
  for( int i=1;i<argc;i++ ){
    if ( argv[i][0] == '-' ){
      if ( strcmp( argv[i],"-h" ) == 0 ){
        if ( ++i<argc && argv[i][0] != '-' ){
          strcpy( exphost,argv[i] );
        }
        else
          i--;
      }
      else if ( strcmp( argv[i],"-i" ) == 0 ){
        while( ++i<argc && argv[i][0] != '-' ){
          iftmp.push_back( argv[i] );
        }
        i--;
      }
      else if ( strcmp( argv[i],"-n" ) == 0 ){
        Nevent = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-skip" ) == 0 ){
        Skipevent = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-trig" ) == 0 ){
        TriggerBit = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-psfile" ) == 0 ){
        Psfile = argv[++i];
      }
      else if ( strcmp( argv[i],"-d" ) == 0 ){
        Debug = 1;
      }
      else{
        std::cout << " unknown option : " << argv[i] << std::endl;
        usage( argc,argv );
        return( 1 );
      }
    }
    else{
      std::cout << " unknown option : " << argv[i] << std::endl;
      usage( argc,argv );
      return( 1 );
    }
  }
  
  // call user init routine
  user_init();

  // main loop
  // connect to experiment
  int  status = cm_connect_experiment( exphost,"",
				       "onlineMonitor",NULL );
  if ( status != CM_SUCCESS )
    return( -1 );
  
  /* open ODB structures */
  RUNINFO_STR(runinfo_str);
  HNDLE hDB, hKey;
  char  str[80];
  cm_get_experiment_database(&hDB, NULL);
  db_create_record(hDB, 0, "/Runinfo", strcomb(runinfo_str));
  db_find_key(hDB, 0, "/Runinfo", &hKey);
  if ( db_open_record(hDB,hKey,&runinfo, sizeof(runinfo),MODE_READ,NULL,NULL)
       != DB_SUCCESS ){
    std::cout << "cannot open /Runinfo." << std::endl;
    return 0;
  }
  
  HNDLE  hbuf;
  int  request_id;
  bm_open_buffer( "SYSTEM",EVENT_BUFFER_SIZE,&hbuf );
  bm_request_event( hbuf, 1, TRIGGER_ALL, GET_SOME,
		    &request_id, user_process_event_exp );
  do{
    status = cm_yield( 1000 );
  } while( status != RPC_SHUTDOWN &&
	   status != SS_ABORT );
  
  cm_disconnect_experiment();
  
  //
  return( 0 );
}


