// -*- C++ -*-
//
// pedestalFilter.cc
//
// Author:  Mitsuhiro YAMAGA
// Created: Wed Aug 28 10:09:54 JST 2002
//
// $Id: pedestalFilter.cc,v 1.1 2002/08/29 14:31:18 yamaga Exp $
//
// Revision History
//
// $Log: pedestalFilter.cc,v $
// Revision 1.1  2002/08/29 14:31:18  yamaga
// Initial version.
//
//

#include <iostream>
#include <list>
#include <string>
#include <stdio.h>
#include <stdlib.h>
#include "MidasManager/MidasManager.h"


// globals
MidasFileIO  ofile;
int          Debug = 0;

double       Nsigma = 2;


//////// user code is defined in the user_ana.cc
extern void  user_init();
extern void  user_begin_of_run( int runnum );
extern int   user_process_event( MidasFileIO& ifile, MidasFileIO& ofile );
extern void  user_end_of_run( int runnum );
extern void  user_term();


////////
// usage
static void
usage( int argc, char* argv[] )
{
  std::cout << "usage : " << argv[0] << " [options]"
            << std::endl 
            << "  options:" << std::endl
            << "    -i <filenames...> (you can use %05d for run_number."
            << " e.g. run%05d.mid)"
            << std::endl 
            << "    -r <run#from> <run#to> (work with %05d)"
            << std::endl
            << "    -n <#event>"
            << std::endl 
            << "    -skip <skipevent>"
            << std::endl
            << "    -nsigma <n> (ADC cut-off < mean + n*sigma)"
            << std::endl
            << "    -o <filename> (you can use %05d for run_number."
            << " e.g. pedsup%05d.mid)"
            << std::endl;
}


////////
// main
int
main( int argc, char* argv[] )
{
  if ( argc < 2 ){
    usage( argc,argv );
    return( 1 );
  }

  // command parser
  int  Skipevent = 0;
  int  Nevent  = -1;
  int  runFrom = 0;
  int  runTo   = 0;
  std::list<std::string>  iflist;
  std::list<std::string>  iftmp;
  std::string  ofname;
  for( int i=1;i<argc;i++ ){
    if ( argv[i][0] == '-' ){
      if ( strcmp( argv[i],"-i" ) == 0 ){
        while( ++i<argc && argv[i][0] != '-' ){
          iftmp.push_back( argv[i] );
        }
        i--;
      }
      else if ( strcmp( argv[i],"-r" ) == 0 ){
        runFrom =  strtol( argv[++i],NULL,0 );
        runTo   =  strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-n" ) == 0 ){
        Nevent = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-skip" ) == 0 ){
        Skipevent = strtol( argv[++i],NULL,0 );
      }
      else if ( strcmp( argv[i],"-nsigma" ) == 0 ){
        Nsigma = strtod( argv[++i],NULL );
      }
      else if ( strcmp( argv[i],"-d" ) == 0 ){
        Debug = 1;
      }
      else if ( strcmp( argv[i],"-o" ) == 0 ){
        ofname = argv[++i];
      }
      else{
        std::cout << " unknown option : " << argv[i] << std::endl;
        usage( argc,argv );
        return( 1 );
      }
    }
    else{
      std::cout << " unknown option : " << argv[i] << std::endl;
      usage( argc,argv );
      return( 1 );
    }
  }
  // iflist
  for( std::list<std::string>::iterator i = iftmp.begin();
       i != iftmp.end(); i++ ){
    if ( strchr( i->c_str(),'%' ) ){
      for( int rn=runFrom; rn<=runTo; rn++ ){
        char  s[1024];
        sprintf( s,i->c_str(),rn );
        iflist.push_back( std::string( s ) );
      }
    }
    else{
      iflist.push_back( *i );
    }
  }
  
  // One output file
  bool  out_each = true;
  if ( !strchr( ofname.c_str(),'%' ) && !ofname.empty() ){
    out_each = false;
    ofile.open( ofname,O_CREAT|O_EXCL|O_RDWR|O_TRUNC|O_LARGEFILE,0644 );
    if ( ofile.good() )
      std::cout << "Opening output file : " << ofname << std::endl;
    else{
      std::cout << "Cannot open output file : " << ofname << std::endl;
      perror( "open" );
    }
  }

  // call user init routine
  user_init();
  
  // main loop
  for( std::list<std::string>::iterator f = iflist.begin();
       f != iflist.end(); f++ ){
    // open data file
    MidasFileIO  mfile( *f,O_RDONLY|O_LARGEFILE );
    if ( mfile.good() ){
      std::cout << "Opening input file : " << *f << std::endl;

      // event bank
      EVENT_HEADER*  pheader = mfile.bank()->eventHeader();
      void*          pevent  = mfile.bank()->eventBuffer();

      // Begin_of_run record
      int  run_number = -1;
      if ( mfile.readBOR() ){
        run_number = pheader->serial_number;
        std::cout << "Run = " << run_number << std::endl;

        // open each output file
        if ( out_each && !ofname.empty() ){
          char  s[1024];
          sprintf( s,ofname.c_str(),run_number );
          ofile.open( s,O_CREAT|O_EXCL|O_RDWR|O_TRUNC|O_LARGEFILE,0644 );
          if ( ofile.good() )
            std::cout << "Opening output file : " << s << std::endl;
          else{
            std::cout << "Cannot open output file : " << s << std::endl;
            perror( "open" );
          }
        }
        // Write BOR
        if ( ofile.good() ){
          MidasBank*  oldbank = ofile.bank( mfile.bank() );
          ofile.writeSpecialEvent();
          ofile.bank( oldbank );
        }
        
        // call user begin_of_run routine
        user_begin_of_run( run_number );
      }

      // Process Event
      mfile.skipEvent( 0,Skipevent );
      for( int i=0; i<Nevent || Nevent<0; i++ ){
        int  id = mfile.readEvent(0);
        if ( (id & 0x8000) == 0x8000 ){
          // Special event (BOR,EOR,MES)
          if ( ofile.good() ){
            MidasBank*  oldbank = ofile.bank( mfile.bank() );
            ofile.writeSpecialEvent();
            ofile.bank( oldbank );
          }
          if ( id == EVENTID_BOR ){
            // call user begin_of_run routine
            user_begin_of_run( run_number );
          }
          else if ( pheader->event_id == EVENTID_EOR ){
            // call user end_of_run routine
            user_end_of_run( run_number );
          }
        }
        else if ( id > 0 ){
          if ( user_process_event( mfile,ofile ) < 0 )
            break;
        }
        else{
          break;
        }
      }

      // Close each output file
      if ( out_each && !ofname.empty() ){
        ofile.close();
      }
    }
    else{
      std::cout << "No such file : " << *f << std::endl;
    }

  }

  // call user term routine
  user_term();

  // close 
  if ( out_each == false && !ofname.empty() ){
    ofile.close();
  }
  
  //
  return( 0 );
}

