// -*- C++ -*-
//
// caenet_client.cc
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Thu Aug 15 23:59:03 JST 2002
//
// $Id: caenet_client.cc,v 1.2 2002/08/30 06:23:33 yamaga Exp $
//
// Revision History
//
// $Log: caenet_client.cc,v $
// Revision 1.2  2002/08/30 06:23:33  yamaga
// updated.
//
// Revision 1.1  2002/08/29 14:06:26  yamaga
// Initial version.
//

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <netdb.h>
#include <strings.h>
#include <unistd.h>
#include <time.h>


//////
void
process_command( int sock,char* command )
{
  printf( "process_command\n" );
  //
  int  cmd = 2;
  int  nw = write( sock,command,strlen(command) );
  //static const struct timespec ts = { 0, 10000 }; // 10 msec
  //nanosleep( &ts,NULL );
  unsigned char  s[1024];
  int  nr = read( sock,s,1024 );
  for( int i=0;i<nr;i+=2 ){
    unsigned short d = (s[i]<<8) | s[i+1];
    printf( "%2d : %04x\n",i/2+1,d );
  }
  char  buf[] = "quit";
  write( sock,buf,strlen(buf) );
}

//////
void
usage()
{
  printf( "usage : caenet_client [-h hostname] [-p port] [--help] command_str\n" );
}


// main
int
main( int argc, char* argv[] )
{
  //
  static int   PORT = 52700;
  static char  hostname[64] = "e391da1";
  char  command[1024]="";
  
  // command argument
  if ( argc > 1 ){
    for( int i=1;i<argc;i++ ){
      if ( strcmp( argv[i],"--help" ) == 0 ){
        usage();
        return( 0 );
      }
      else if ( strcmp( argv[i],"-h" ) == 0 ){
        if ( ++i < argc )
          strcpy( hostname,argv[i] );
      }
      else if ( strcmp( argv[i],"-p" ) == 0 ){
        if ( ++i < argc )
          PORT = strtol( argv[i],NULL,0 );
      }
      else if ( argv[i][0] == '-' ){
        usage();
        return( 0 );
      }
      else{
        // command for CAEN SY527
        strncpy( command,argv[i],1023 );
        command[strlen(command)] = '\n';
      }
    }
  }
  if ( strlen( command ) < 1 ){
    usage();
    return( 0 );
  }

  // init
  struct hostent  *serverhost;
  struct sockaddr_in  server;
  int  sock;
  
  if ( (serverhost = gethostbyname( hostname ) ) == NULL ){
    fprintf( stderr,"Bad host name : %s\n", hostname );
    return( -1 );
  }
  
  bzero( (char*)&server, sizeof(server) );
  server.sin_family     = AF_INET;
  server.sin_port       = htons(PORT);
  server.sin_addr.s_addr= ((struct in_addr*)serverhost->h_addr)->s_addr;
  fprintf( stderr,"addr = %08x\n",server.sin_addr.s_addr );
  
  if ( (sock = socket( AF_INET,SOCK_STREAM,0 ) ) < 0 ){
    fprintf( stderr,"Socket for caenetd failed. sock=%d\n",sock );
    perror( "socket" );
    return( -1 );
  }
  
  if ( connect( sock,(const sockaddr *)&server,sizeof(server) ) == -1 ){
    fprintf( stderr,"Cannot connect to caenet server %s port %d.\n",
             hostname,PORT );
    perror( "connect" );
    return( -1 );
  }
  // connected
  process_command( sock,command );
  
  //
  return( 0 );
}

