// -*- C++ -*-
//
// smp_tko.cc
// utility
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Mon Apr  8 21:14:27 JST 2002
//
// $Id: smp_tko.cc,v 1.1.1.1 2002/08/13 12:48:12 kensh Exp $
//
// Revision History
//
// $Log: smp_tko.cc,v $
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <iomanip>
#include <signal.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include "midas.h"
#include "mcstd.h"

#include "smp/Smp.h"
#include "vmeioreg/VmeIoreg.h"
#include "smp_tko.h"


//
static int    debugLevel = 0;

// START flag
static bool  Started = false;

// TKO-TDC-SMP
static const int  Nsmp = 2;
static const unsigned int  SMP_BASEADDR[Nsmp] = {0x11000000,0x12000000};
static int  ma_tdc[Nsmp][10] = { { 2, 4, 6, 8,10,12,14,16 },
                                 { 3, 5, 7, 9,11,13,15,17 } };
static Smp  smp[Nsmp];

static char* banknameSMP[] = { "SMP0","SMP1" };
static char* banknameTDC[] = { "TDC0","TDC1" };

// VME-IOREG
static const unsigned int  VIOREG_BASEADDR  = 0x8000;
VmeIoreg  vioreg( VIOREG_BASEADDR );

// Interrupt Handler
bool  Interrupted = false;
extern"C"
void
sighdl( int arg )
{
  Interrupted = true;
  if ( debugLevel ){
    std::cerr << "Interrupt received. arg=" << arg << std::endl;
  }
}


////////////////
void
initSmpTko()
{
  if ( debugLevel )
    std::cerr << "initSmp" << std::endl;
  
  // Disable interrupt
  vioreg.disableInt();
  vioreg.clear();
  vioreg.mask();
  vioreg.level_out( 0x00 );
  
  // VME interrupt setup
  if ( !VmeIoreg::initVuiInterrupt( 0,5,0x50 ) ){
    abort();
  }
  sigset( SIGINT,sighdl );

  // Init TKO-SMP
  int  istat;
  for( int i=0; i<Nsmp; i++ ){
    // init SMP-TKO
    smp[i].init( SMP_BASEADDR[i],0*debugLevel );
    
    // reset SMP
    smp[i].tkoCommand( SMP_CMR,SMP_RSET );
    istat = smp[i].tkoStatus( SMP_STAR );
    printf( "TKO[%d] STAR = 0x%08x\n",i,istat );
    
    // stop sequence 
    smp[i].tkoCommand( SMP_CMR, SMP_SEQ_NOP );
    
    // clear counter
    smp[i].tkoCommand( SMP_SNCCR, 0x01 );
    istat = smp[i].tkoStatus( SMP_SNCCR );
    printf( "TKO[%d] SNCCR = 0x%08x\n",i,istat );
    
    // Buffer Switch Enable
    //smp[i].tkoCommand( SMP_BCR, SMP_BCR_SWENABLE | SMP_BCR_SWMASK );
    //istat = smp[i].tkoStatus( SMP_BCR );
    //printf( "TKO[%d] BCR = 0x%08x\n",i,istat );
    
    // set Pedestal to SCH
    smp[i].tkoCommand( SMP_CMR,SMP_TKOF(9) );
    for( int ch=0; ch<2047; ch++ ){
      smp[i].tkoWrite( 24,ch,0,0xfff );
    }
  }
  // Set SMP interrupt 
  smp[0].tkoCommand( SMP_ICR,0x25 );
  istat = smp[0].tkoStatus( SMP_ICR );
  printf( "TKO[0] ICR = 0x%08x\n",istat );
  
  smp[0].tkoCommand( SMP_IVR,0x50 );
  istat = smp[0].tkoStatus( SMP_IVR );
  printf( "TKO[0] IVR = 0x%08x\n",istat );

}


////////////////
void
termSmpTko()
{
  if ( debugLevel )
    std::cerr << "termNgf" << std::endl;

  // disable interrupt
  vioreg.disableInt();

  // stop sequence 
  for( int i=0;i<Nsmp;i++ ){
    smp[i].tkoCommand( SMP_CMR, SMP_SEQ_NOP );
  }

  // close
  VmeIoreg::termVuiInterrupt();
  sigset( SIGINT,SIG_DFL );
}


////////////////
void
beginRunSmpTko()
{
  if ( debugLevel )
    std::cerr << "beginRunSmpTko" << std::endl;
  
  // Buffer_switch enable
  for( int i=0; i<Nsmp; i++ ){
    smp[i].tkoCommand( SMP_BCR, SMP_BCR_SWENABLE | SMP_BCR_SWMASK );
  }

  // Force Buffer_switch 
  Interrupted = false;
  int  ct = 0;
  do {
    vioreg.out( 0x10 ); // hardware switch
    static const struct timespec ts = { 0, 1000000 }; // 1 msec
    nanosleep( &ts,NULL );
    if ( ++ct > 100 ){
      std::cerr << "beginSmpTko : Buffer Switch retry=" << ct 
                << std::endl << std::endl
                << "***" << std::endl
                << "*** PLEASE RESTART THIS FRONTEND. " << std::endl
                << "***" 
                << std::endl << std::endl;
      abort();
    }
  } while( !Interrupted );
  if ( ct > 1 )
    std::cerr << "beginRunSmpTko : Buffer Switch retry=" << ct << std::endl;
  
  // Clear Interrupt, Enable Interrupt.
  Interrupted = false;
  vioreg.enableInt();

  // Wait until all frontends are ready.
  Started = false;
  vioreg.clear();
  vioreg.level_out( 0x80 );
  //std::cerr << "Waiting other frontends ready." << std::endl;

  // Do sequence 
  for( int i=0; i<Nsmp; i++ ){
    smp[i].tkoCommand( SMP_CMR, SMP_SEQ_AC_CMP );
  }

}


////////////////
void
endRunSmpTko()
{
  if ( debugLevel )
    std::cerr << "endRunSmpTko" << std::endl;

  // Disable interrupt
  vioreg.disableInt();
  vioreg.level_out( 0x00 );

  // Stop sequence 
  for( int i=0;i<Nsmp;i++ ){
    smp[i].tkoCommand( SMP_CMR, SMP_SEQ_NOP );
  }
  
  // Buffer Switch Autoclear
  for( int i=0;i<Nsmp;i++ ){
    smp[i].tkoCommand( SMP_BCR, 
                       SMP_BCR_SWENABLE | SMP_BCR_SWMASK | SMP_BCR_SWAUTOCLR );
  }

  // Force Buffer Switch 
  Interrupted = false;
  int  ct = 0;
  do {
    vioreg.out( 0x10 ); // hardware switch
    static const struct timespec ts = { 0, 1000000 }; // 1 msec
    nanosleep( &ts,NULL );
    if ( ++ct > 100 ){
      std::cerr << "endRunSmpTko : Buffer Switch retry=" << ct 
                << std::endl << std::endl
                << "***" << std::endl
                << "*** PLEASE RESTART THIS FRONTEND. " << std::endl
                << "***" 
                << std::endl << std::endl;
      abort();
    }
  } while( !Interrupted );
  if ( ct > 1 )
    std::cerr << "endRunSmpTko : Buffer Switch retry=" << ct << std::endl;
  Interrupted = false;

  // stop
  Started = false;
}


////////////////
void
pauseRunSmpTko()
{
  if ( debugLevel )
    std::cerr << "pauseRunSmpTko" << std::endl;
 
  // disable interrupt
  vioreg.disableInt();
  vioreg.level_out( 0x00 );

  // stop sequence 
  for( int i=0;i<Nsmp;i++ ){
    smp[i].tkoCommand( SMP_CMR, SMP_SEQ_NOP );
  }

#if 0
  // Buffer Switch Autoclear
  for( int i=0;i<Nsmp;i++ ){
    smp[i].tkoCommand( SMP_BCR,
                       SMP_BCR_SWENABLE | SMP_BCR_SWMASK | SMP_BCR_SWAUTOCLR );
  }

  // Force Buffer Switch 
  Interrupted = false;
  int  ct = 0;
  do {
    vioreg.out( 0x10 ); // hardware switch
    static const struct timespec ts = { 0, 1000000 }; // 1 msec
    nanosleep( &ts,NULL );
    if ( ++ct > 100 ){
      std::cerr << "pauseRunSmpTko : Buffer Switch retry=" << ct 
                << std::endl << std::endl
                << "***" << std::endl
                << "*** PLEASE RESTART THIS FRONTEND. " << std::endl
                << "***" 
                << std::endl << std::endl;
      abort();
    }
  } while( !Interrupted );
  if ( ct > 1 )
    std::cerr << "pauseRunSmpTko : Buffer Switch retry=" << ct << std::endl;
  Interrupted = false;
#endif

  // stop
  Started = false;
}


////////////////
void
resumeRunSmpTko()
{
  if ( debugLevel )
    std::cerr << "resumeRunSmpTko" << std::endl;
 
  // Buffer_switch enable
  for( int i=0; i<Nsmp; i++ ){
    smp[i].tkoCommand( SMP_BCR, SMP_BCR_SWENABLE | SMP_BCR_SWMASK );
  }

  // Force Buffer Switch 
  Interrupted = false;
  int  ct = 0;
  do {
    vioreg.out( 0x10 ); // hardware switch
    static const struct timespec ts = { 0, 1000000 }; // 1 msec
    nanosleep( &ts,NULL );
    if ( ++ct > 100 ){
      std::cerr << "beginSmpTko : Buffer Switch retry=" << ct 
                << std::endl << std::endl
                << "***" << std::endl
                << "*** PLEASE RESTART THIS FRONTEND. " << std::endl
                << "***" 
                << std::endl << std::endl;
      abort();
    }
  } while( !Interrupted );
  if ( ct > 1 )
    std::cerr << "beginRunSmpTko : Buffer Switch retry=" << ct << std::endl;
  Interrupted = false;
  
  // Clear Interrupt, Enable Interrupt.
  Interrupted = false;
  vioreg.enableInt();

  // Wait until all frontends are ready.
  Started = false;
  vioreg.clear();
  vioreg.level_out( 0x80 );
  //std::cerr << "Waiting other frontends ready." << std::endl;

  // Do sequence 
  for( int i=0; i<Nsmp; i++ ){
    smp[i].tkoCommand( SMP_CMR, SMP_SEQ_AC_CMP );
  }

}


////////////////
void
fillBankSmpTko( char*& pevent )
{
  if ( debugLevel )
    std::cerr << "fillBankSmpTko" << std::endl;
  
  // Only in the start sequence.
  if ( !Started ){
    vioreg.level_out( 0x00 );
    vioreg.out( 0x01 );
    Started = true;
    //std::cerr << "SMP-TKO started" << std::endl;
    return;
  }
  
  // 
  Interrupted = false;

  if ( debugLevel ){
    int  stat = smp[0].tkoStatus( SMP_BCR );
    printf( "fillBankSmpTko : TKO[%d] BCR = 0x%08x\n",0,stat );
  }

  for( int i=0;i<Nsmp;i++ ){
    // SMP bank
    DWORD *pdata;
    bk_create(pevent, banknameSMP[i], TID_DWORD, &pdata);
    *pdata++ = smp[i].tkoStatus( SMP_BCR );
    *pdata++ = smp[i].tkoStatus( SMP_ENR );
    *pdata++ = smp[i].tkoStatus( SMP_DSR );
    bk_close(pevent, pdata);
  
    /* create TDC bank */
    bk_create(pevent, banknameTDC[i], TID_DWORD, &pdata);
    
    // Read event
    if ( debugLevel ){
      int  istat = smp[i].tkoStatus( SMP_BCR );
      printf( "TKO[%d] BCR = 0x%08x\n",i,istat );
      istat = smp[i].tkoStatus( SMP_ENR ) & 0xffff;
      printf( "ENR status = 0x%08x (%d events)\n",istat,istat );  
    }
    // Data size
    int  dsr = smp[i].tkoStatus( SMP_DSR );
    int  ds  = dsr & 0x00ffffff;
    int  overflow = dsr & SMP_DSR_OVERFLOW;
    if ( debugLevel ){
      printf( "DSR status = 0x%08x (%d bytes)\n",dsr,ds );  
    }
    // Block read
    smp[i].smpReadD( 0,ds );
    const unsigned int* tdc = smp[i].dmaPtr();
    int  ds2 = ds >> 2;
    for ( int n=0; n<ds2; n++ )
      *pdata++ = tdc[n];
    if ( ds2 > 0 )
      *pdata++ = 0xffffffff; // terminator
    
    if ( debugLevel ){
      for( int d=0; d<ds2; d++ ){
        printf( "TDC[%d][%d] = 0x%08x\n",i,d,tdc[d] );
      }
    }
    
    // close TDC bank
    bk_close(pevent, pdata);
  }

  // fast clear signal for ADC/TDC by VME IOreg
  Interrupted = false;
  vioreg.clear();

}
