// -*- C++ -*-
//
// E391Detector.h
//
// Author:  Mitsuhiro YAMAGA
// Created: Tue Aug  6 09:22:10 JST 2002
//
// $Id: E391Detector.h,v 1.4 2002/10/14 07:51:14 yamaga Exp $
//
// Revision History
//
// $Log: E391Detector.h,v $
// Revision 1.4  2002/10/14 07:51:14  yamaga
// Added trigger-bit macro definition.
//
// Revision 1.3  2002/09/27 12:07:16  yamaga
// CosmicCounter and some functions added.
//
// Revision 1.2  2002/08/29 13:43:43  yamaga
// pbank() modified to bank()
//
// Revision 1.1.1.1  2002/08/13 12:48:13  kensh
//
//
// Start version.
//
//

#ifndef E391DETECTOR_H_INCLUDED
#define E391DETECTOR_H_INCLUDED

//
#include "midas.h"
#include "unpack/AdcEvent.h"
#include "unpack/TdcEvent.h"
#include "detparam/DetParam.h"
#include "detector/Detector.h"

// Trigger Bit of TRG1
#define  TRIG_ONSPILL  0x01
#define  TRIG_SPILLON  0x01
#define  TRIG_CLOCK    0x02
#define  TRIG_COSMIC   0x04
#define  TRIG_XE       0x08
#define  TRIG_NCLUS    0x10

// Number of channels
#define N_MAX_C6              2
#define N_MAX_FRONTBARREL    32
#define N_MAX_CC01            8
#define N_MAX_CC02            8
#define N_MAX_MAINBARREL    192
#define N_MAX_CHARGEVETO     36
#define N_MAX_CSI           576  // normal 496 + deformed 56 + KTeV CsI 24 
#define N_MAX_SANDWICH       24
#define N_MAX_CC03            6
#define N_MAX_CC04            4
#define N_MAX_CC05            4
#define N_MAX_BEAMCATCHER    40
#define N_MAX_COSMICCOUNTER   6

#define N_MAX_C6_SUM              1
#define N_MAX_FRONTBARREL_SUM     4
#define N_MAX_CC01_SUM            1
#define N_MAX_CC02_SUM            1
#define N_MAX_MAINBARREL_SUM     24
#define N_MAX_CHARGEVETO_SUM      5
#define N_MAX_CSI_SUM            72
#define N_MAX_SANDWICH_SUM        4
#define N_MAX_CC03_SUM            1
#define N_MAX_CC04_SUM            1
#define N_MAX_CC05_SUM            1
#define N_MAX_BEAMCATCHER_SUM     5
#define N_MAX_COSMICCOUNTER_SUM   1

//
class E391Detector{
public:
  // singleton
  static E391Detector&  instance();

  // extractor
  inline const Detector&  C6( int id ) const;
  inline const Detector&  FrontBarrel( int id ) const;
  inline const Detector&  CC01( int id ) const;
  inline const Detector&  CC02( int id ) const;
  inline const Detector&  MainBarrel( int id ) const;
  inline const Detector&  ChargeVeto( int id ) const;
  inline const Detector&  CsI( int id ) const;
  inline const Detector&  Sandwich( int id ) const;
  inline const Detector&  CC03( int id ) const;
  inline const Detector&  CC04( int id ) const;
  inline const Detector&  CC05( int id ) const;
  inline const Detector&  BeamCatcher( int id ) const;
  inline const Detector&  CosmicCounter( int id ) const;
  
  inline const Detector&  C6_Sum( int id ) const;
  inline const Detector&  FrontBarrel_Sum( int id ) const;
  inline const Detector&  CC01_Sum( int id ) const;
  inline const Detector&  CC02_Sum( int id ) const;
  inline const Detector&  MainBarrel_Sum( int id ) const;
  inline const Detector&  ChargeVeto_Sum( int id ) const;
  inline const Detector&  CsI_Sum( int id ) const;
  inline const Detector&  Sandwich_Sum( int id ) const;
  inline const Detector&  CC03_Sum( int id ) const;
  inline const Detector&  CC04_Sum( int id ) const;
  inline const Detector&  CC05_Sum( int id ) const;
  inline const Detector&  BeamCatcher_Sum( int id ) const;
  inline const Detector&  CosmicCounter_Sum( int id ) const;
  
  inline const AdcEvent&  adcCrate( int crate ) const;
  inline const TdcEvent&  tdcCrate( int crate ) const;

  inline const AdcEvent*  pAdcCrate( int crate ) const;
  inline const TdcEvent*  pTdcCrate( int crate ) const;
  
  //
  int     runNumber() const { return m_runNumber; }
  int     spillNumber() const;
  int     eventNumber() const;
  int     eventTime() const;
  int     triggerBit( int nth=1 ) const;

  int     bank( const std::string& bankname, void* pdata ) const;
  bool    exist( const std::string& bankname ) const;
  
  void    setRunNumber( int rn ) { m_runNumber = rn; }
  int     getDB( int run_number=0 );

  void    setEvent( EVENT_HEADER* pheader, void* pevent ); 
  int     unpack( int nth=1 );
  int     unpack( EVENT_HEADER* pheader, void* pevent, int nth=1 ); 
 
protected:
  E391Detector();
  ~E391Detector();

private:
  // singleton
  static E391Detector*  s_e391det;

  // detparam
  DetParam*  m_detparam;
  
  // detector
  int  N_C6;
  int  N_FRONTBARREL;
  int  N_CC01;
  int  N_CC02;
  int  N_MAINBARREL;
  int  N_CHARGEVETO;
  int  N_CSI;
  int  N_SANDWICH;
  int  N_CC03;
  int  N_CC04;
  int  N_CC05;
  int  N_BEAMCATCHER;
  int  N_COSMICCOUNTER;

  int  N_C6_SUM;
  int  N_FRONTBARREL_SUM;
  int  N_CC01_SUM;
  int  N_CC02_SUM;
  int  N_MAINBARREL_SUM;
  int  N_CHARGEVETO_SUM;
  int  N_CSI_SUM;
  int  N_SANDWICH_SUM;
  int  N_CC03_SUM;
  int  N_CC04_SUM;
  int  N_CC05_SUM;
  int  N_BEAMCATCHER_SUM;
  int  N_COSMICCOUNTER_SUM;

  Detector  m_C6[ N_MAX_C6 ];
  Detector  m_FrontBarrel[ N_MAX_FRONTBARREL ];
  Detector  m_CC01[ N_MAX_CC01 ];
  Detector  m_CC02[ N_MAX_CC02 ];
  Detector  m_MainBarrel[ N_MAX_MAINBARREL ];
  Detector  m_ChargeVeto[ N_MAX_CHARGEVETO ];
  Detector  m_CsI[ N_MAX_CSI ];
  Detector  m_Sandwich[ N_MAX_SANDWICH ];
  Detector  m_CC03[ N_MAX_CC03 ];
  Detector  m_CC04[ N_MAX_CC04 ];
  Detector  m_CC05[ N_MAX_CC05 ];
  Detector  m_BeamCatcher[ N_MAX_BEAMCATCHER ];
  Detector  m_CosmicCounter[ N_MAX_COSMICCOUNTER ];

  Detector  m_C6_Sum[ N_MAX_C6_SUM ];
  Detector  m_FrontBarrel_Sum[ N_MAX_FRONTBARREL_SUM ];
  Detector  m_CC01_Sum[ N_MAX_CC01_SUM ];
  Detector  m_CC02_Sum[ N_MAX_CC02_SUM ];
  Detector  m_MainBarrel_Sum[ N_MAX_MAINBARREL_SUM ];
  Detector  m_ChargeVeto_Sum[ N_MAX_CHARGEVETO_SUM ];
  Detector  m_CsI_Sum[ N_MAX_CSI_SUM ];
  Detector  m_Sandwich_Sum[ N_MAX_SANDWICH_SUM ];
  Detector  m_CC03_Sum[ N_MAX_CC03_SUM ];
  Detector  m_CC04_Sum[ N_MAX_CC04_SUM ];
  Detector  m_CC05_Sum[ N_MAX_CC05_SUM ];
  Detector  m_BeamCatcher_Sum[ N_MAX_BEAMCATCHER_SUM ];
  Detector  m_CosmicCounter_Sum[ N_MAX_COSMICCOUNTER_SUM ];

  const static Detector  s_dummydet; // dummy
  const static AdcEvent  s_dummyAdcEvent; // dummy
  const static TdcEvent  s_dummyTdcEvent; // dummy

  // data
  AdcEvent*  m_adcCrate[MAX_NADCCRATE];
  TdcEvent*  m_tdcCrate[MAX_NTDCCRATE];

  int            m_runNumber;
  EVENT_HEADER*  m_pheader;
  void*          m_pevent;

  // method
  void    build();
  void    init_param();
  void    delete_param();
};

// inline
#include "E391Detector/E391Detector.icc"


#endif // DETECTOR_H_INCLUDED

