// -*- C++ -*-
//
// TMNtuple.cc
// NTUPLE accessing class
//
// Author:  Mitsuhiro YAMAGA (yamaga@post.kek.jp)
// Created: Tue Sep 11 17:17:42 JST 2001
//
// $Id: TMNtuple.cc,v 1.2 2002/08/29 13:49:31 yamaga Exp $
//
// Revision History
//
// $Log: TMNtuple.cc,v $
// Revision 1.2  2002/08/29 13:49:31  yamaga
// delete [] m_varname corrected.
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <string>
#include <string.h>
#include "ccfortran/ccfortran.h"
#include "TupleManager/TupleManager.h"
#include "TupleManager/TMNtuple.h"


//
extern int  quest_[100]; // common/quest/iquest(100)


//////// TMNtuple
//
TMNtuple::TMNtuple( const TupleManager* mn )
  : m_bufinit( false ),
    m_type( 0 ),
    m_manager( mn ),
    m_title( "" ),
    m_block( "" ),
    m_cname( "" ),
    m_hid( 0 ),
    m_nvars( 0 ),
    m_vars( 0 ),
    m_varindex( 0 ),
    m_varname( 0 )
{

}


//
TMNtuple::~TMNtuple()
{
  if ( m_type == TM_RWNNAME ){
    delete    m_vars;
    delete    m_varindex;
    delete [] m_varname;
  }
}


// CWN
void
TMNtuple::bookCWN( int hid, const std::string& title,
                   const std::string& block,
                   void* vars, const std::string& cname )
{
  // delete old setting
  if ( m_type == TM_RWNNAME ){
    delete    m_vars;
    delete    m_varindex;
    delete [] m_varname;
    m_vars     = 0;
    m_varindex = 0;
    m_varname  = 0;
  }

  //
  m_type  = TM_CWN;
  m_hid   = hid;
  m_title = title;
  m_block = block;
  m_vars  = (float*)vars;
  m_nvars = 0;
  m_cname = cname;

  if ( m_manager->status() == TM_Write ){
    int  err = 0;
    HBSET( "BSIZE",8192,err );
    HCDIR( m_manager->cdir(), " " );
    HBNT( m_hid, m_title, "D" );
    HBNAME( m_hid, m_block, m_vars, m_cname );
  }
  else if ( m_manager->status() == TM_Read ){
    HRIN( m_hid,999,0 ); // dummy read     
  }
  
}


// CWN, read only
void
TMNtuple::bookCWN( int hid, const std::string& block, void* vars )
{
  // delete old setting
  if ( m_type == TM_RWNNAME ){
    delete    m_vars;
    delete    m_varindex;
    delete [] m_varname;
    m_vars     = 0;
    m_varindex = 0;
    m_varname  = 0;
  }

  m_type  = TM_CWN;
  m_hid   = hid;
  m_title = "";
  m_block = block;
  m_vars  = (float*)vars;
  m_cname = "";

  HRIN( m_hid,999,0 ); // dummy read 

  if ( m_manager->status() == TM_Write ){
    std::cout << "TMNtuple::bookCWN : Data name and type not defined."
              << std::endl;
  }
  
}


// RWN by vars
void
TMNtuple::bookRWN( int hid, const std::string& title,
                   void* vars, const std::string& varname )
{
  // delete old setting
  if ( m_type == TM_RWNNAME ){
    delete    m_vars;
    delete    m_varindex;
    delete [] m_varname;
    m_vars     = 0;
    m_varindex = 0;
    m_varname  = 0;
  }

  m_type  = TM_RWN;
  m_hid   = hid;
  m_title = title;
  m_block = "";
  m_vars  = (float*)vars;
  m_nvars = 0;
  m_cname = "";

  // varname
  char*  svar = new char[varname.length()+1];
  strcpy( svar,varname.c_str() );
  char* cp   = svar;
  //char* next = cp;
  //cp = strsep( &next," \n" );
  cp = strtok( cp," \n" );
  m_nvars = 0;
  do{
    m_cname += std::string( cp + std::string( "        " ),0,8 );
    m_nvars++;
    //cp = strsep( &next," \n" );
    cp = strtok( NULL," \n" );
  } while( cp != NULL );

  // book
  HBOOKN( hid,title,m_nvars,m_manager->dir(),81920,m_cname );

  delete svar;
}


// RWN by name
void
TMNtuple::bookRWN( int hid, const std::string& title,
                   const std::string& varname )
{
  // delete old setting
  if ( m_type == TM_RWNNAME ){
    delete    m_vars;
    delete    m_varindex;
    delete [] m_varname;
    m_vars     = 0;
    m_varindex = 0;
    m_varname  = 0;
  }

  m_type  = TM_RWNNAME;
  m_hid   = hid;
  m_title = title;
  m_block = "";
  m_cname = "";

  // varname
  char*  svar = new char[varname.length()+1];
  strcpy( svar,varname.c_str() );
  char* cp   = svar;
  //char* next = cp;
  //cp = strsep( &next," ,\n" );
  cp = strtok( cp," ,\n" );
  std::list<std::string> clist;
  m_nvars = 0;
  while( cp != NULL ){
    clist.push_back( cp );
    m_cname += std::string( cp + std::string( "        " ),0,8 );
    m_nvars++;
    //cp = strsep( &next," ,\n" );
    cp = strtok( NULL," ,\n" );
  };

  // book
  HBOOKN( hid,title,m_nvars,m_manager->dir(),81920,m_cname );

  // index
  m_varname  = new std::string[m_nvars];
  m_varindex = new int[m_nvars];
  m_vars     = new float[m_nvars];
  
  if ( m_varname && m_varindex && m_vars ){
    std::list<std::string>  ctmp = clist;
    ctmp.sort();
    int  ii=0;
    for( std::list<std::string>::iterator i=clist.begin();
         i!=clist.end(); i++ ){
      m_varname[ii++] = *i;
    }
    ii = 0;
    for( std::list<std::string>::iterator j=ctmp.begin();
         j!=ctmp.end(); j++ ){
      for( int i=0;i<m_nvars;i++ ){
        if ( m_varname[i] == *j ){
          m_varindex[ii++] = i;
          break;
        }
      }
    }
    clear();
    
    //for( int i=0;i<m_nvars;i++ ){
    //  std::cout << m_varname[m_varindex[i]] << std::endl;
    //}
    
  }
  else{
    std::cerr << "Cannot allocate variables" << std::endl;
    delete    m_vars;
    delete    m_varindex;
    delete [] m_varname;
    m_vars     = 0;
    m_varindex = 0;
    m_varname  = 0;
  }

  //
  delete svar;
}


// RWN by vars, read only
void
TMNtuple::bookRWN( int hid, void* vars )
{
  m_type  = TM_RWN;
  m_hid   = hid;
  m_title = "";
  m_block = "";
  m_vars  = (float*)vars;
  m_cname = "";
}


//
int
TMNtuple::dump()
{
  if ( m_type == TM_CWN ){// CWN
    HFNT( m_hid );
    //HFNTB( m_hid,m_block );
  }
  else if ( m_type == TM_RWN || m_type == TM_RWNNAME ){ //RWN
    HFN( m_hid,m_vars );
  }
  
  return( quest_[0] );
}


//
void
TMNtuple::fill( const std::string& vname, float val )
{
  if ( m_type == TM_RWNNAME ){
    // find vname
    int  ni = 0;
    int  ne = m_nvars;
    while( 1 ){
      int  nm = (ni + ne) / 2;
      //std::cout << "VNAME=" << vname
      //          << " NVARS=" << m_nvars << " n=" << nm 
      //          << " NAME=" << m_varname[m_varindex[nm]]
      //          << std::endl;
      if ( m_varname[m_varindex[nm]] > vname ){
        if ( ne > nm )
          ne = nm;
        else
          break; // cannot find
      }
      else if ( m_varname[m_varindex[nm]] < vname ){
        if ( ni < nm )
          ni = nm;
        else
          break; // cannot find
      }
      else{
        // found
        m_vars[m_varindex[nm]] = val;
        return;
      }
    }
    std::cout << "TMNtuple::fill : cannot find the name " << vname
              << std::endl;
  }
  else{
    std::cout << "TMNtuple::fill : this is not a RWN by name."
              << std::endl;
  }
}


//
void
TMNtuple::clear()
{
  if ( m_type == TM_RWNNAME ){
    for( int i=0;i<m_nvars;i++ ){
      m_vars[i] = 0;
    }
  }
  else{
    std::cout << "TMNtuple::clear : this is not a RWN by name."
              << std::endl;
  }
}



//
int
TMNtuple::read( int eventNo )
{
  int  ierr;
  
  if ( m_type == TM_CWN ){
    if ( !m_bufinit ){
      HCDIR( m_manager->cdir(), " " );
      HBNAME( m_hid, " ", 0 ,"$CLEAR" );
      HBNAME( m_hid, m_block, m_vars, "$SET" );
      HGNT( m_hid, eventNo, ierr );
      m_bufinit = true;
    }
    else{
      HGNTF( m_hid, eventNo, ierr );
    }
  }
  else if ( m_type == TM_RWN ){
    if ( !m_bufinit ){
      HGNPAR( m_hid,"TMNtuple::read" );
      m_bufinit = true;
    }
    HGNF( m_hid, eventNo, m_vars, ierr );
  }

  return( ierr );
}


//
int
TMNtuple::totalEntries()
{
  int  ent;

  HCDIR( m_manager->cdir(), " " );
  HNOENT( m_hid, ent );

  return( ent );
}
