// -*- C++ -*- 
//
// hbmerge.cc
// Histogram utiliity
//
// author  : Mitsuhiro YAMAGA
// created : Tue May 22 01:37:45 JST 2001
//
// $Id: hbmerge.cc,v 1.2 2002/09/03 14:18:17 yamaga Exp $
//
// Revision history
//
// $Log: hbmerge.cc,v $
// Revision 1.2  2002/09/03 14:18:17  yamaga
// Conform to updated HMERGE
//
// Revision 1.1.1.1  2002/08/13 12:48:12  kensh
//
//
// Start version.
//
//

#include <iostream>
#include <iomanip>
#include <list>
#include <string>
#include "ccfortran/ccfortran.h"


////////////////////////////////
// constants
const int  hlim = 10000000;


////////////////////////////////
// fortran
extern "C" {
  int  pawc_[hlim];
}


////////////////////////////////
// variables
static std::list<std::string> g_ifname;
static std::string            g_ofname;
static std::string            g_ihpss = "";
static std::string            g_ohpss = "";


////////////////////////////////
// functions

// usage
static void
usage()
{
  std::cout << "usage : hbmerge [-ohpss] [-o ofile] [-ihpss] [inputfiles...]"
            << std::endl;
  exit( 0 );
}

// parce option
static int
parse_option( int argc, const char* argv[] )
{
  // welcome
  //std::cout << "hbmerge : parse_option()" << std::endl;

  //
  if ( argc < 2 ){
    usage();
  }
  
  //
  int  nopt = 0;
  for( int i=1; i<argc; i++ ){
    // help
    if ( strcmp( argv[i], "-h" ) == 0 ||
         strcmp( argv[i], "--help" ) == 0 ){
      usage();
    }

    // HPSS
    else if ( strcmp( argv[i], "-ohpss" ) == 0 ) {
      g_ohpss = "/hpss:";
      std::cout << "hbmerge : parce_option() : ohpss = " 
                << g_ohpss
                << std::endl;
      nopt++;
    }
    else if ( strcmp( argv[i], "-ihpss" ) == 0 ) {
      g_ihpss = "/hpss:";
      std::cout << "hbmerge : parce_option() : ihpss = " 
                << g_ihpss
                << std::endl;
      nopt++;
    }

    // output file
    else if ( strcmp( argv[i], "-o" ) == 0 ){
      g_ofname = argv[++i];
      std::cout << "hbmerge : parce_option() : output file = " 
                << g_ofname
                << std::endl;
      nopt++;
    }

    // input file
    else {
      g_ifname.push_back( std::string(argv[i]) );
      std::cout << "hbmerge : parce_option() : input file = " 
                << *g_ifname.rbegin()
                << std::endl;
      nopt++;
    }

  }

  //
  return( nopt );
}


////////////////////////////////
// HROPEN_LARGE
extern "C"
void hropen_large_( int* lun, const char* dir, const char* fname, 
                   const char* opt, int maxrec, int* stat,
                    int ldir, int lfname, int lopt )
{
  std::string  sdir( dir,ldir );
  std::string  sfname( fname,lfname );
  std::string  sopt( opt,lopt );
  sopt += "CX";
  //std::cout << *lun << std::endl
  //          << sdir << std::endl
  //          << sfname << std::endl
  //          << sopt << std::endl;
  HROPEN( *lun, sdir, sfname, sopt, 1000000, *stat );
}


////////////////////////////////
// main
int
main( int argc, const char* argv[] )
{
  // welcome
  //std::cout << "hbmerge : main()" << std::endl;

  // command parser
  parse_option( argc,argv );
  
  // init HBOOK
  HLIMIT( hlim );

  // hpss option
  g_ofname = g_ohpss + g_ofname;
  for( std::list<std::string>::iterator i = g_ifname.begin();
       i != g_ifname.end(); i++ ){
    *i = g_ihpss + *i;
  }
  
  // call modified hmerge
  HMERGE( g_ifname, g_ofname );

  //
  return( 0 );
}
